#include <Arduino.h>
#include <Wire.h>

//define functions for bit manipulation
#define set(dir,pin) (dir |= pin)
#define clear(dir,pin) (dir &= (~pin))
#define get(dir, pin) (dir & pin) > 0

//define multiplex
#define PORTD_MULTIPLEX_MASK 0x9E
#define PORTB_MULTIPLEX_MASK 0x03
const uint8_t digits[11]={
  0x3F, //0
  0x06, //1
  0x5B, //2
  0x4F, //3
  0x66, //4
  0x6D, //5
  0x7D, //6
  0x07, //7
  0x7F, //8
  0x6F, //9
  0x00, //clear digit
};

#define DIGIT0 (1<<PB3)
#define DIGIT1 (1<<PB4)
#define DIGIT2 (1<<PB5)
#define DIGIT3 (1<<PC2)
#define DIGIT4 (1<<PC3)
#define DIGIT012_port PORTB
#define DIGIT012_dir DDRB
#define DIGIT34_port PORTC
#define DIGIT34_dir DDRC

volatile uint8_t displayDigits[5]={0,2,6,0,8};
volatile uint8_t currentDigit = 0;
void Multiplex_init();
void displayNum(uint8_t num);


void setup() {
  // put your setup code here, to run once:
  //CLKPR – Clock Prescale Register changed to divide clock signal by 1
  CLKPR = (1<<CLKPCE); //10000000
  CLKPR = (0<<CLKPS3)|(0<<CLKPS2)|(0<<CLKPS1)|(0<<CLKPS0); //00000000
  
  // Serial.begin(9600);
  Wire.begin();
  Multiplex_init();
}
void DisplayInt(int value){
  if(value>9999) return;
  uint8_t buffer[4] = {10,10,10,10};
  for (uint8_t i = 0; i < 4; i++)
  {
    buffer[i] = value%10;
    value = value /10;
  }
  //Set the display digits
  displayDigits[1] = buffer[3];
  displayDigits[2] = buffer[2];
  displayDigits[3] = buffer[1];
  displayDigits[4] = buffer[0];
}
void loop() {
  // put your main code here, to run repeatedly:
  int bytesRecived = Wire.requestFrom(4,2);
  if(bytesRecived == 0){
    displayDigits[1] = 10;
    displayDigits[2] = 10;
    displayDigits[3] = 10;
    displayDigits[4] = 10;
  }
  // Serial.print(bytesRecived);
  // Serial.print(" ");
  while(Wire.available()){
    byte low = Wire.read();
    byte high = Wire.read();
    int lightValue = low | (high<<8);
    DisplayInt(lightValue);
    // Serial.println(lightValue);
  }
  
  delay(100);
}

//Display
void Multiplex_init(){

    set(DDRD,PORTD_MULTIPLEX_MASK);
    set(DDRB,PORTB_MULTIPLEX_MASK);

    set(DIGIT012_dir,DIGIT0|DIGIT1|DIGIT2);
    set(DIGIT34_dir,DIGIT3|DIGIT4);
    set(DIGIT012_port,DIGIT0|DIGIT1|DIGIT2);
    set(DIGIT34_port,DIGIT3|DIGIT4);
    cli();
    TCCR2A = (0<<COM1A1) | (0<<WGM21) | (0<<WGM20); // set PWM fast mode on pin 11
    TCCR2B = (1<<CS22) | (0<<CS21) | (0<<CS20); //set prescaler to 1/128 for multiplaxin
    OCR2A = 255;
    TIMSK2 |= (1<<TOIE2); // enable overflow interrupt on timer 2 for multiplex
    sei();
}
ISR(TIMER2_OVF_vect){
    set(DIGIT012_port,DIGIT0|DIGIT1|DIGIT2);
    set(DIGIT34_port,DIGIT3|DIGIT4);
    displayNum(displayDigits[currentDigit]);

    switch (currentDigit)
    {
    case 0:clear(DIGIT012_port,DIGIT0); break;
    case 1:clear(DIGIT012_port,DIGIT1); break;
    case 2:clear(DIGIT012_port,DIGIT2); break;
    case 3:clear(DIGIT34_port,DIGIT3); break;
    case 4:clear(DIGIT34_port,DIGIT4); break;
    default: break;
    }
    ++currentDigit;
    currentDigit %= 5;
}
void displayNum(uint8_t num){
    clear(PORTD,PORTD_MULTIPLEX_MASK);
    clear(PORTB,PORTB_MULTIPLEX_MASK);

    uint8_t digit = digits[num];
    uint8_t portD = ((PORTD_MULTIPLEX_MASK & ~(1<<PD7) ) & (digit<<2)) | ((1<<PD7)&(digit<<4)) | ((1<<PD1)&(digit>>5));
    PORTD |= PORTD_MULTIPLEX_MASK&portD;
    PORTB |= PORTB_MULTIPLEX_MASK&(digit>>4);
}